from typing import Optional, Union, List

from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ImportDispLineStyleTuple(Tuple):
    """Import Display Line Style Tuple"""

    __tupleType__ = diagramTuplePrefix + "ImportDispLineStyleTuple"

    name: str = TupleField()

    #: Fill the dash space, or let the what ever is underneath show.
    backgroundFillDashSpace: bool = TupleField()

    CAP_BUTT = "butt"
    CAP_ROUND = "round"
    CAP_SQUARE = "square"
    capStyle: str = TupleField()

    JOIN_BEVEL = "bevel"
    JOIN_ROUND = "round"
    JOIN_MITER = "miter"
    joinStyle: str = TupleField()

    startArrowSize: float = TupleField()
    endArrowSize: float = TupleField()

    winStyle: str = TupleField()

    #: A dash pattern style
    # We accept :
    # * None
    # an integer
    # a list of integers
    dashPattern: Optional[Union[List[int], int]] = TupleField()

    importHash: str = TupleField()

    modelSetKey: str = TupleField()



from typing import List, Any, Optional

import json
import json
from vortex import SerialiseUtil
from vortex.Tuple import Tuple, addTupleType, TupleField

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.worker.tasks.LookupHashConverter import (
    LookupHashConverter,
)
from peek_plugin_diagram.tuples.branches.ImportBranchTuple import ImportBranchTuple


@addTupleType
class BranchTuple(Tuple):
    """Branch Tuple

    This is the private branch tuple used to work with the branch.

    """

    __ID_NUM = 0
    __COORD_SET_ID_NUM = 1
    __KEY_NUM = 2
    __VISIBLE_NUM = 3
    __UPDATED_DATE = 4
    __CREATED_DATE = 5
    __DISPS_NUM = 6
    __ANCHOR_DISP_KEYS_NUM = 7
    __UPDATED_BY_USER_NUM = 8
    __NEEDS_SAVE_NUM = 9  # Not stored in DB
    __LAST_EDIT_POSITION = 10
    __LAST_INDEX_NUM = 10

    __tupleType__ = diagramTuplePrefix + "BranchTuple"

    __rawJonableFields__ = ["packedJson__"]

    #:  The packed JSON data for this object
    packedJson__: List[Any] = TupleField([])

    # This field is server side only
    importHash: str = TupleField()
    importGroupHash: str = TupleField()

    def __init__(self, **kwargs):
        Tuple.__init__(self, **kwargs)
        self.packedJson__ = [None] * (BranchTuple.__LAST_INDEX_NUM + 1)

    @classmethod
    def loadFromImportTuple(
        cls,
        importBranchTuple: ImportBranchTuple,
        coordSetId: int,
        lookupHashConverter: LookupHashConverter,
    ) -> "BranchTuple":
        """Load From Import Tuple

        This is used by the import worker to pack this object into the index.

        """
        raise NotImplementedError("BranchTuple.loadFromImportTuple")
        # deltasJson = []
        # for importDelta in importBranchTuple.deltas:
        #     delta = BranchDeltaBase.loadFromImportTuple(
        #         importDeltaTuple=importDelta,
        #         lookupHashConverter=lookupHashConverter
        #     )
        #     deltasJson.append(delta._jsonData)

        self = cls()
        self.packedJson__ = [
            # TODO, Create the packed json array
        ]
        self.importHash = importBranchTuple.importHash
        self.importGroupHash = importBranchTuple.importGroupHash
        return self

    def packJson(self) -> str:
        return json.dumps(self.packedJson__)

    @classmethod
    def loadFromJson(
        self, packedJsonStr: str, importHash: str, importGroupHash: str
    ) -> "BranchTuple":
        branchTuple = BranchTuple()
        branchTuple.packedJson__ = json.loads(packedJsonStr)
        branchTuple.importHash = importHash
        branchTuple.importGroupHash = importGroupHash

        while len(branchTuple.packedJson__) < BranchTuple.__LAST_INDEX_NUM + 1:
            branchTuple.packedJson__.append(None)

        return branchTuple

    def __array(self, num):
        if self.packedJson__[num] is None:
            self.packedJson__[num] = []
        return self.packedJson__[num]

    @property
    def id(self):
        return self.packedJson__[self.__ID_NUM]

    @id.setter
    def id(self, value: Optional[int]):
        self.packedJson__[self.__ID_NUM] = value

    @property
    def coordSetId(self):
        return self.packedJson__[self.__COORD_SET_ID_NUM]

    @property
    def key(self):
        return self.packedJson__[self.__KEY_NUM]

    @property
    def updatedByUser(self) -> str:
        return self.packedJson__[self.__UPDATED_BY_USER_NUM]

    @updatedByUser.setter
    def updatedByUser(self, val: str) -> None:
        self.packedJson__[self.__UPDATED_BY_USER_NUM] = val

    @property
    def disps(self) -> List:
        return self.__array(self.__DISPS_NUM)

    @disps.setter
    def disps(self, disps: List) -> None:
        self.packedJson__[self.__DISPS_NUM] = disps

    @property
    def anchorDispKeys(self) -> List[str]:
        return self.__array(self.__ANCHOR_DISP_KEYS_NUM)

    @property
    def visible(self) -> bool:
        return self.packedJson__[self.__VISIBLE_NUM]

    @property
    def updatedDate(self):
        if self.packedJson__[self.__UPDATED_DATE] is None:
            return None
        return SerialiseUtil.fromStr(
            self.packedJson__[self.__UPDATED_DATE], SerialiseUtil.T_DATETIME
        )

    @property
    def createdDate(self):
        return SerialiseUtil.fromStr(
            self.packedJson__[self.__CREATED_DATE], SerialiseUtil.T_DATETIME
        )

    @property
    def lastEditPositionJsonStr(self) -> str:
        return self.packedJson__[self.__LAST_EDIT_POSITION]



import logging

from sqlalchemy import Column, Index, BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import (
    ACIProcessorQueueTupleABC,
)
from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.storage.DeclarativeBase import DeclarativeBase

logger = logging.getLogger(__name__)


@addTupleType
class BranchIndexCompilerQueue(Tuple, DeclarativeBase, ACIProcessorQueueTupleABC):
    __tablename__ = "BranchIndexCompilerQueue"
    __tupleType__ = diagramTuplePrefix + "BranchIndexCompilerQueueTable"

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    modelSetId = Column(
        Integer,
        ForeignKey("ModelSet.id", ondelete="CASCADE"),
        nullable=False,
        autoincrement=True,
    )

    chunkKey = Column(String, primary_key=True)

    __table_args__ = (
        Index(
            "idx_BICompQueue_modelSetId_chunkKey", modelSetId, chunkKey, unique=False
        ),
    )

    @classmethod
    def sqlCoreLoad(cls, row):
        return BranchIndexCompilerQueue(
            id=row.id, modelSetId=row.modelSetId, chunkKey=row.chunkKey
        )

    @property
    def ckiUniqueKey(self):
        return self.chunkKey



import logging
from datetime import datetime
from typing import List

import pytz
from sqlalchemy import select, and_

from peek_plugin_diagram._private.storage.DispIndex import DispIndexerQueue
from peek_plugin_diagram._private.storage.LiveDbDispLink import LiveDbDispLink
from peek_plugin_diagram._private.storage.ModelSet import ModelSet, ModelCoordSet
from peek_plugin_diagram._private.worker.tasks.LiveDbDisplayValueConverter import (
    LiveDbDisplayValueConverter,
)
from peek_plugin_livedb.tuples.LiveDbDisplayValueTuple import LiveDbDisplayValueTuple

logger = logging.getLogger(__name__)


class WorkerApiImpl:
    """Worker Api

    This class allows other classes to work with the Diagram plugin on the
    worker service.

    """

    _FETCH_SIZE = 5000

    @classmethod
    def updateLiveDbDisplayValues(
        cls,
        ormSession,
        modelSetKey: str,
        liveDbRawValues: List[LiveDbDisplayValueTuple],
    ) -> None:
        if not liveDbRawValues:
            return

        startTime = datetime.now(pytz.utc)

        modelSetId = (
            ormSession.query(ModelSet.id).filter(ModelSet.key == modelSetKey).one().id
        )

        translater = LiveDbDisplayValueConverter.create(ormSession, modelSetId)

        for item in liveDbRawValues:
            if item.dataType is None:
                logger.warning("LiveDB key %s is missing dataType", item.key)
                continue

            item.displayValue = translater.translate(item.dataType, item.rawValue)

        logger.debug(
            "Converted %s LiveDB Raw Values in %s",
            len(liveDbRawValues),
            (datetime.now(pytz.utc) - startTime),
        )

    @classmethod
    def liveDbDisplayValueUpdateNotify(
        cls, ormSession, modelSetKey: str, updatedKeys: List[str]
    ):

        logger.debug("TODO TODO - liveDbDisplayValueUpdateNotify coordSetId")
        linkTable = LiveDbDispLink.__table__
        coordSetTable = ModelCoordSet.__table__
        queueTable = DispIndexerQueue.__table__

        modelSetId = (
            ormSession.query(ModelSet.id).filter(ModelSet.key == modelSetKey).one().id
        )

        stmt = (
            select([linkTable.c.dispId])
            .select_from(
                linkTable.join(
                    coordSetTable, linkTable.c.coordSetId == coordSetTable.c.id
                )
            )
            .where(
                and_(
                    linkTable.c.liveDbKey.in_(updatedKeys),
                    coordSetTable.c.modelSetId == modelSetId,
                )
            )
        )

        ins = queueTable.insert().from_select(["dispId"], stmt)
        ormSession.execute(ins)



from typing import List

from vortex.Tuple import TupleField, addTupleType, Tuple

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix


@addTupleType
class ImportBranchDeltaCreateDisp(Tuple):
    """Imported Branch Delta Create Disp

    This branch delta will change the color of disps.

    """

    __tupleType__ = diagramTuplePrefix + "ImportBranchDeltaCreateDisp"

    #:  A list of keys to apply this color to.
    disps: List[str] = TupleField([])



import logging

from peek_plugin_base.worker.PluginWorkerEntryHookABC import PluginWorkerEntryHookABC
from peek_plugin_diagram._private.worker.tasks import (
    GridCompilerTask,
    ImportDispTask,
    DispCompilerTask,
    LocationIndexCompilerTask,
)
from peek_plugin_diagram._private.worker.tasks.branch import (
    BranchIndexCompiler,
    BranchIndexImporter,
    BranchIndexUpdater,
)

logger = logging.getLogger(__name__)


class WorkerEntryHook(PluginWorkerEntryHookABC):
    def load(self):
        logger.debug("loaded")

    def start(self):
        logger.debug("started")

    def stop(self):
        logger.debug("stopped")

    def unload(self):
        logger.debug("unloaded")

    @property
    def celeryAppIncludes(self):
        return [
            BranchIndexUpdater.__name__,
            BranchIndexCompiler.__name__,
            BranchIndexImporter.__name__,
            DispCompilerTask.__name__,
            GridCompilerTask.__name__,
            ImportDispTask.__name__,
            LocationIndexCompilerTask.__name__,
        ]



"""Added branch fields to DispBase

Peek Plugin Database Migration Script

Revision ID: 4f6ed1047562
Revises: 361988cd327d
Create Date: 2019-05-11 15:05:14.096453

"""

# revision identifiers, used by Alembic.
revision = "4f6ed1047562"
down_revision = "361988cd327d"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispBase",
        sa.Column("branchId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "DispBase",
        sa.Column("branchStage", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "DispBase", sa.Column("hashId", sa.String(), nullable=True), schema="pl_diagram"
    )
    op.add_column(
        "DispBase",
        sa.Column("replacesHashId", sa.String(), nullable=True),
        schema="pl_diagram",
    )
    op.create_index(
        "idx_Disp_branchId", "DispBase", ["branchId"], unique=False, schema="pl_diagram"
    )
    op.create_foreign_key(
        "DispBase_branchId_fkey",
        "DispBase",
        "BranchIndex",
        ["branchId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
        ondelete="CASCADE",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "DispBase_branchId_fkey", "DispBase", schema="pl_diagram", type_="foreignkey"
    )
    op.drop_index("idx_Disp_branchId", table_name="DispBase", schema="pl_diagram")
    op.drop_column("DispBase", "replacesHashId", schema="pl_diagram")
    op.drop_column("DispBase", "hashId", schema="pl_diagram")
    op.drop_column("DispBase", "branchStage", schema="pl_diagram")
    op.drop_column("DispBase", "branchId", schema="pl_diagram")
    # ### end Alembic commands ###



"""Added DispNull

Peek Plugin Database Migration Script

Revision ID: 6d3a96cf1955
Revises: aa76b059b81e
Create Date: 2019-05-29 10:51:00.872234

"""

# revision identifiers, used by Alembic.
revision = "6d3a96cf1955"
down_revision = "aa76b059b81e"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "DispNull",
        sa.Column("id", sa.Integer(), autoincrement=False, nullable=False),
        sa.Column("geomJson", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["pl_diagram.DispBase.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_diagram",
    )


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("DispNull", schema="pl_diagram")
    # ### end Alembic commands ###



import logging
from typing import Union, Optional

from twisted.internet.defer import Deferred, inlineCallbacks
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_diagram._private.client.controller.GridCacheController import (
    GridCacheController,
)
from peek_plugin_diagram._private.storage.ModelSet import makeDispGroupGridKey
from peek_plugin_diagram._private.tuples.GroupDispsTuple import GroupDispsTuple
from peek_plugin_diagram._private.tuples.grid.EncodedGridTuple import EncodedGridTuple

logger = logging.getLogger(__name__)


class ClientGroupDispsTupleProvider(TuplesProviderABC):
    def __init__(self, gridCacheController: GridCacheController):
        self.gridCacheController = gridCacheController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        coordSetId = tupleSelector.selector["coordSetId"]
        gridKey = makeDispGroupGridKey(coordSetId)
        grid: Optional[EncodedGridTuple] = self.gridCacheController.encodedChunk(
            gridKey
        )

        groupDispTuple = GroupDispsTuple()
        groupDispTuple.coordSetId = coordSetId
        groupDispTuple.encodedGridTuple = grid.encodedGridTuple if grid else None

        payloadEnvelope = yield Payload(
            filt, tuples=[groupDispTuple]
        ).makePayloadEnvelopeDefer()

        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg



from sqlalchemy import Column, Index, ForeignKey, DateTime, BigInteger
from sqlalchemy import Integer, String
from sqlalchemy.orm import relationship
from vortex.Tuple import Tuple, addTupleType

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.storage.DeclarativeBase import DeclarativeBase
from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet


@addTupleType
class BranchIndex(Tuple, DeclarativeBase):
    """Branch Index

    The BranchIndex is implemented to allow editing of individual branches via the UI,
    AND staging of imported branches so they are packed into the grids.

    This table stores the JSON packed versions of the branches, ready to be compiled
    into the encoded chunk.

    This table is not in an ideal format for updates (appending deltas, etc), but since
    edits are performed by users, this can be unpacked and re-packed suitable fast.

    """

    __tablename__ = "BranchIndex"
    __tupleType__ = diagramTuplePrefix + "BranchIndexTable"

    #:  The unique ID of this branchIndex (database generated)
    id = Column(BigInteger, primary_key=True, autoincrement=True)

    #:  The model set for this branchIndex
    coordSetId = Column(
        Integer, ForeignKey("ModelCoordSet.id", ondelete="CASCADE"), nullable=False
    )
    coordSet = relationship(ModelCoordSet)

    importHash = Column(String)

    importGroupHash = Column(String)

    #:  The unique key of this branchIndex
    key = Column(String, nullable=False)

    #:  The chunk that this branchIndex fits into
    chunkKey = Column(String, nullable=False)

    updatedDate = Column(DateTime(timezone=True))
    createdDate = Column(DateTime(timezone=True), nullable=False)

    #:  The JSON ready for the Compiler to use
    packedJson = Column(String, nullable=False)

    __table_args__ = (
        Index("idx_BranchIndex_key", coordSetId, key, unique=True),
        Index("idx_BranchIndex_chunkKey", chunkKey, unique=False),
        Index("idx_BranchIndex_importHash", coordSetId, importHash, unique=True),
        Index(
            "idx_BranchIndex_importGroupHash", coordSetId, importGroupHash, unique=False
        ),
    )



"""
 * orm.ModelSet.py
 *
 *  Copyright Synerty Pty Ltd 2011
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by
 *  Synerty Pty Ltd
 *
"""
from sqlalchemy.exc import IntegrityError

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from sqlalchemy.orm import relationship
from sqlalchemy.sql.schema import Index
from sqlalchemy.sql.sqltypes import Boolean
from sqlalchemy.types import Float
from vortex.Tuple import addTupleType, Tuple, TupleField

from .DeclarativeBase import DeclarativeBase


@addTupleType
class ModelSet(Tuple, DeclarativeBase):
    __tablename__ = "ModelSet"
    __tupleType__ = diagramTuplePrefix + __tablename__

    id = Column(Integer, primary_key=True, autoincrement=True)
    key = Column(String(50), nullable=False)
    name = Column(String(50), nullable=False)
    comment = Column(String)

    landingCoordSetId = Column(
        Integer, ForeignKey("ModelCoordSet.id", ondelete="CASCADE"), nullable=True
    )

    coordSets = relationship(
        "ModelCoordSet",
        remote_side="ModelCoordSet.modelSetId",
        primaryjoin="ModelSet.id==ModelCoordSet.modelSetId",
    )

    __table_args__ = (Index("idx_ModelSet_name", name, unique=True),)


@addTupleType
class ModelCoordSet(Tuple, DeclarativeBase):
    """Coordinate Sets"""

    __tablename__ = "ModelCoordSet"
    __tupleType__ = diagramTuplePrefix + __tablename__

    # Ensure gridSizes is serialised when it's sent to the client
    __fieldNames__ = ["gridSizes"]

    id = Column(Integer, primary_key=True, autoincrement=True)
    key = Column(String(50), nullable=False)
    name = Column(String(50), nullable=False)
    initialPanX = Column(Float, nullable=False, server_default="0")
    initialPanY = Column(Float, nullable=False, server_default="0")
    initialZoom = Column(Float, nullable=False, server_default="2.0")
    positionOnZoom = Column(Float, nullable=False, server_default="2.0")

    enabled = Column(Boolean, nullable=False, server_default="false")

    modelSetId = Column(
        Integer, ForeignKey("ModelSet.id", ondelete="CASCADE"), nullable=False
    )
    modelSet = relationship(
        ModelSet,
        foreign_keys=[modelSetId],
        primaryjoin="ModelSet.id==ModelCoordSet.modelSetId",
    )

    # Grid size settings
    gridSizes = relationship(
        "ModelCoordSetGridSize", lazy="subquery", order_by="ModelCoordSetGridSize.key"
    )

    minZoom = Column(Float, nullable=False, server_default="0.01")
    maxZoom = Column(Float, nullable=False, server_default="10.0")

    multiplierX = Column(Float, nullable=False, server_default="1")
    multiplierY = Column(Float, nullable=False, server_default="1")

    offsetX = Column(Float, nullable=False, server_default="0")
    offsetY = Column(Float, nullable=False, server_default="0")

    comment = Column(String)

    importId1 = Column(Integer)
    importId2 = Column(String(100))

    #: Misc data holder
    data = TupleField()
    isLanding = TupleField()

    #: Show this Coord Set as a group of DispGroups to choose from in the Editor
    dispGroupTemplatesEnabled = Column(Boolean, nullable=False, server_default="false")

    #: Show this Coord Set as a group of Line Templates to choose from in the Editor
    edgeTemplatesEnabled = Column(Boolean, nullable=False, server_default="false")

    #: Is Editing enabled? (Also ensure ALL editDefault fields are set.
    branchesEnabled = Column(Boolean, nullable=False, server_default="false")

    #: Is Editing enabled? (Also ensure ALL editDefault fields are set.
    editEnabled = Column(Boolean, nullable=False, server_default="false")

    #: Default Layer for new shapes
    editDefaultLayerId = Column(Integer, ForeignKey("DispLayer.id"))

    #: Default Level for new shapes
    editDefaultLevelId = Column(Integer, ForeignKey("DispLevel.id"))

    #: Default Color for new shapes
    editDefaultColorId = Column(Integer, ForeignKey("DispColor.id"))

    #: Default Line for new shapes
    editDefaultLineStyleId = Column(Integer, ForeignKey("DispLineStyle.id"))

    #: Default Text for new shapes
    editDefaultTextStyleId = Column(Integer, ForeignKey("DispTextStyle.id"))

    #: Default Vertex/Node/Equipment Coord Set
    editDefaultVertexCoordSetId = Column(Integer, ForeignKey("ModelCoordSet.id"))
    editDefaultVertexGroupName = Column(String)

    #: Default Edge/Conductor Coord Set
    editDefaultEdgeCoordSetId = Column(Integer, ForeignKey("ModelCoordSet.id"))
    editDefaultEdgeGroupName = Column(String)

    __table_args__ = (
        Index("idxCoordSetModelName", modelSetId, name, unique=True),
        Index("idxCoordSetImportId1", importId1, unique=False),
        Index("idxCoordSetImportId2", importId2, unique=False),
        Index("idxCoordModelSetId", modelSetId, unique=False),
        Index("idxCoordModel_editDefaultLayerId", editDefaultLayerId, unique=False),
        Index("idxCoordModel_editDefaultLevelId", editDefaultLevelId, unique=False),
        Index("idxCoordModel_editDefaultColorId", editDefaultColorId, unique=False),
        Index(
            "idxCoordModel_editDefaultLineStyleId", editDefaultLineStyleId, unique=False
        ),
        Index(
            "idxCoordModel_editDefaultTextStyleId", editDefaultTextStyleId, unique=False
        ),
        Index(
            "idxCoordModel_editDefaultVertexCoordSetId",
            editDefaultVertexCoordSetId,
            unique=False,
        ),
        Index(
            "idxCoordModel_editDefaultEdgeCoordSetId",
            editDefaultEdgeCoordSetId,
            unique=False,
        ),
    )


@addTupleType
class ModelCoordSetGridSize(Tuple, DeclarativeBase):
    """Coord Set Grid Size Settings

    To match a Z_GRID the display item must be min <= ON < max
    The equal to is on the min side, not the max side

    """

    __tablename__ = "ModelCoordSetGridSize"
    __tupleType__ = diagramTuplePrefix + __tablename__

    id = Column(Integer, primary_key=True, autoincrement=True)
    key = Column(Integer, nullable=False)
    min = Column(Float, nullable=False)
    max = Column(Float, nullable=False)
    xGrid = Column(Integer, nullable=False)
    yGrid = Column(Integer, nullable=False)

    smallestTextSize = Column(Float, nullable=False, server_default="6.0")
    smallestShapeSize = Column(Float, nullable=False, server_default="2.0")

    coordSetId = Column(
        Integer, ForeignKey("ModelCoordSet.id", ondelete="CASCADE"), nullable=False
    )
    coordSet = relationship(ModelCoordSet)

    __table_args__ = (Index("idx_CoordSetGridSize_key", coordSetId, key, unique=True),)

    DEFAULT = [
        # These defaults are good for Aurora
        dict(
            min=0.0,
            max=0.04,
            key=0,
            xGrid=30000,
            yGrid=30000,
            smallestShapeSize=20,
            smallestTextSize=8,
        ),
        dict(
            min=0.04,
            max=0.1,
            key=1,
            xGrid=10000,
            yGrid=10000,
            smallestShapeSize=20,
            smallestTextSize=8,
        ),
        dict(
            min=0.1,
            max=0.5,
            key=2,
            xGrid=2000,
            yGrid=2000,
            smallestShapeSize=20,
            smallestTextSize=8,
        ),
        dict(
            min=0.5,
            max=1000.0,
            key=3,
            xGrid=1000,
            yGrid=1000,
            smallestShapeSize=2,
            smallestTextSize=4,
        ),
    ]

    def makeGridKey(self, x, y):
        """Make Grid Key

        coordSetId = ModelCoordSet.id
        gridSize = GridSize (above)
        x, y = Grid coordinates, top left
        """
        return "%s|%s.%sx%s" % (self.coordSetId, self.key, x, y)


def makeDispGroupGridKey(coordSetId: int):
    """Make Disp Group Grid Key

    Make the special disp group grid key name.
    This is used to store all of the DispGroups that are not specifically stored in a
    grid, with the DispGroupPtr that uses it.

    """
    return "%s|dispgroup" % coordSetId


def getOrCreateModelSet(session, modelSetKey):
    qry = session.query(ModelSet).filter(ModelSet.key == modelSetKey)
    if not qry.count():
        session.add(ModelSet(name=modelSetKey, key=modelSetKey))
        try:
            session.commit()
        except IntegrityError:
            return qry.one()

    return qry.one()


def getOrCreateCoordSet(session, modelSetKey, coordSetKey):
    modelSet = getOrCreateModelSet(session, modelSetKey)

    qry = (
        session.query(ModelCoordSet)
        .filter(ModelCoordSet.modelSetId == modelSet.id)
        .filter(ModelCoordSet.key == coordSetKey)
    )

    if not qry.count():
        coordSet = ModelCoordSet(
            modelSetId=modelSet.id, name=coordSetKey, key=coordSetKey
        )
        session.add(coordSet)

        for gridSize in ModelCoordSetGridSize.DEFAULT:
            newGrid = ModelCoordSetGridSize(**gridSize)
            newGrid.coordSet = coordSet
            session.add(newGrid)

        try:
            session.commit()
        except IntegrityError:
            return qry.one()

    return qry.one()



"""Remove unique constraint for DispGroup

Peek Plugin Database Migration Script

Revision ID: 6f545d4166dc
Revises: 6a71c73c0606
Create Date: 2019-06-03 23:15:20.007675

"""

# revision identifiers, used by Alembic.
revision = "6f545d4166dc"
down_revision = "6a71c73c0606"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "DispGroup_name_key", "DispGroup", schema="pl_diagram", type_="unique"
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "DispGroup_name_key", "DispGroup", ["name"], schema="pl_diagram"
    )
    # ### end Alembic commands ###



"""added coordset mx c

Peek Plugin Database Migration Script

Revision ID: 4b943b584307
Revises: d103b9c1eba9
Create Date: 2017-10-01 11:16:34.264774

"""

# revision identifiers, used by Alembic.
revision = "4b943b584307"
down_revision = "d103b9c1eba9"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    op.add_column(
        "ModelCoordSet",
        sa.Column("multiplierX", sa.Float(), server_default="1", nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "ModelCoordSet",
        sa.Column("multiplierY", sa.Float(), server_default="1", nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "ModelCoordSet",
        sa.Column("offsetX", sa.Float(), server_default="0", nullable=True),
        schema="pl_diagram",
    )
    op.add_column(
        "ModelCoordSet",
        sa.Column("offsetY", sa.Float(), server_default="0", nullable=True),
        schema="pl_diagram",
    )

    op.execute(
        """UPDATE "pl_diagram"."ModelCoordSet"
                  SET "multiplierX" = 1,
                        "multiplierY" = 1,
                        "offsetX" = 0,
                        "offsetY" = 0
                       """
    )

    op.alter_column(
        "ModelCoordSet",
        "multiplierX",
        type_=sa.Float,
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "ModelCoordSet",
        "multiplierY",
        type_=sa.Float,
        nullable=False,
        schema="pl_diagram",
    )
    op.alter_column(
        "ModelCoordSet", "offsetX", type_=sa.Float, nullable=False, schema="pl_diagram"
    )
    op.alter_column(
        "ModelCoordSet", "offsetY", type_=sa.Float, nullable=False, schema="pl_diagram"
    )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("ModelCoordSet", "offsetY", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "offsetX", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "multiplierY", schema="pl_diagram")
    op.drop_column("ModelCoordSet", "multiplierX", schema="pl_diagram")
    # ### end Alembic commands ###



"""Added polygon isRectangle

Peek Plugin Database Migration Script

Revision ID: 6a71c73c0606
Revises: e8d6d02c0922
Create Date: 2019-06-03 22:12:09.814648

"""

# revision identifiers, used by Alembic.
revision = "6a71c73c0606"
down_revision = "e8d6d02c0922"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DispPolygon",
        sa.Column("isRectangle", sa.Boolean(), nullable=True),
        schema="pl_diagram",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DispPolygon", "isRectangle", schema="pl_diagram")
    # ### end Alembic commands ###



from typing import List

from abc import ABCMeta, abstractmethod
from twisted.internet.defer import Deferred


class DiagramImportApiABC(metaclass=ABCMeta):
    @abstractmethod
    def importDisps(
        self,
        modelSetKey: str,
        coordSetKey: str,
        importGroupHash: str,
        dispsEncodedPayload: bytes,
    ) -> Deferred:
        """Import Disps

        Add or replace display items in a model

        :param modelSetKey:  The name of the model set to import the disps into
        :param coordSetKey:  The name of the cooridinate set to import the disps into
        :param importGroupHash:  The unique hash of the input group to import into
        :param dispsEncodedPayload: An array of disps to import, wrapped in a serialised
                    payload.


        Wrap the disps list with ::

                dispsEncodedPayload = Payload(tuples=disps).toEncodedPayload()


        :return: A deferred that fires when the disps are loaded and queued for compile

        """

    @abstractmethod
    def importLookups(
        self,
        modelSetKey: str,
        coordSetKey: str,
        lookupTupleType: str,
        lookupTuples: List,
        deleteOthers: bool = True,
        updateExisting: bool = True,
    ) -> Deferred:
        """Import Lookups

        Add or replace diplay lookups in a model

        :param modelSetKey:  The name of the model set to import the lookups into
        :param coordSetKey:  The name of the coord set to import the lookups into
        :param lookupTupleType:  The type of lookups being imported
        :param lookupTuples: An array of the lookups
        :param deleteOthers: Delete existing lookups that are not present in lookupTuples
        :param updateExisting: If a lookup already exists, update it.
                This is matched by the "importHash"

        :return: A deferred that fires when the lookups are imported

        """

    @abstractmethod
    def getLookups(
        self, modelSetKey: str, coordSetKey: str, lookupTupleType: str
    ) -> Deferred:
        """Get Lookups

        Use this method to retrieve lookups that have been previously imported.

        :param modelSetKey:  The name of the model set for the lookups
        :param coordSetKey:  The name of the coord set for the lookups
        :param lookupTupleType:  The type of lookups to return

        :return: A deferred that fires with a list of lookup tuples. These tuples
                are the same type used during the import.

        """



from typing import Dict

from peek_abstract_chunked_index.private.tuples.ACIUpdateDateTupleABC import (
    ACIUpdateDateTupleABC,
)
from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import addTupleType, TupleField, Tuple

#: This the type of the data that we get when the clients observe new locationIndexs.
DeviceLocationIndexT = Dict[str, str]


@addTupleType
class LocationIndexUpdateDateTuple(Tuple, ACIUpdateDateTupleABC):
    __tupleType__ = diagramTuplePrefix + "LocationIndexUpdateDateTuple"

    # Improve performance of the JSON serialisation
    __rawJonableFields__ = ("initialLoadComplete", "updateDateByChunkKey")

    initialLoadComplete: bool = TupleField()
    updateDateByChunkKey: DeviceLocationIndexT = TupleField({})

    @property
    def ckiUpdateDateByChunkKey(self):
        return self.updateDateByChunkKey



import json
import logging
from datetime import datetime
from typing import List, Dict

import pytz
from peek_plugin_base.storage.DbConnection import pgCopyInsert

from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_base.worker.CeleryDbConn import prefetchDeclarativeIds
from peek_plugin_diagram._private.storage.LiveDbDispLink import (
    LiveDbDispLink,
    LIVE_DB_KEY_DATA_TYPE_BY_DISP_ATTR,
)
from peek_plugin_diagram._private.storage.ModelSet import ModelCoordSet
from peek_plugin_diagram.tuples.model.ImportLiveDbDispLinkTuple import (
    ImportLiveDbDispLinkTuple,
)
from peek_plugin_livedb.tuples.ImportLiveDbItemTuple import ImportLiveDbItemTuple

logger = logging.getLogger(__name__)


def importDispLinks(
    coordSet: ModelCoordSet,
    importGroupHash: str,
    importDispLinks: List[ImportLiveDbDispLinkTuple],
) -> List[ImportLiveDbItemTuple]:
    """Import Disps Links

    1) Drop all disps with matching importGroupHash

    2) set the  coordSetId

    :param coordSet:
    :param importGroupHash:
    :param importDispLinks: An array of import LiveDB Disp Links to import
    :return:
    """
    dispLinkTable = LiveDbDispLink.__table__
    dispLinkIdIterator = prefetchDeclarativeIds(LiveDbDispLink, len(importDispLinks))

    startTime = datetime.now(pytz.utc)

    ormSession = CeleryDbConn.getDbSession()
    try:

        ormSession.execute(
            dispLinkTable.delete().where(
                dispLinkTable.c.importGroupHash == importGroupHash
            )
        )

        if not importDispLinks:
            return []

        liveDbItemsToImportByKey = {}

        dispLinkInserts = []

        for importDispLink in importDispLinks:
            dispLink = _convertImportDispLinkTuple(coordSet, importDispLink)
            dispLink.id = next(dispLinkIdIterator)

            liveDbItem = _makeImportLiveDbItem(importDispLink, liveDbItemsToImportByKey)

            dispLink.liveDbKey = liveDbItem.key
            dispLinkInserts.append(dispLink.tupleToSqlaBulkInsertDict())

        # if dispLinkInserts:
        #     ormSession.execute(LiveDbDispLink.__table__.insert(), dispLinkInserts)

        ormSession.commit()

        if dispLinkInserts:
            # This commits it's self
            rawConn = CeleryDbConn.getDbEngine().raw_connection()
            pgCopyInsert(rawConn, LiveDbDispLink.__table__, dispLinkInserts)
            rawConn.commit()

        logger.info(
            "Inserted %s LiveDbDispLinks in %s",
            len(dispLinkInserts),
            (datetime.now(pytz.utc) - startTime),
        )

        return list(liveDbItemsToImportByKey.values())

    finally:
        ormSession.close()


def _convertImportDispLinkTuple(
    coordSet: ModelCoordSet, importDispLink: ImportLiveDbDispLinkTuple
) -> LiveDbDispLink:
    return LiveDbDispLink(
        dispId=importDispLink.internalDispId,
        # Dynamically added in DispImportController
        coordSetId=coordSet.id,
        dispAttrName=importDispLink.dispAttrName,
        liveDbKey=importDispLink.liveDbKey,
        importGroupHash=importDispLink.importGroupHash,
        propsJson=json.dumps(importDispLink.props),
    )


def _makeImportLiveDbItem(
    importDispLink: ImportLiveDbDispLinkTuple, liveDbItemsToImportByKey: Dict
):
    if importDispLink.liveDbKey in liveDbItemsToImportByKey:
        return liveDbItemsToImportByKey[importDispLink.liveDbKey]

    dataType = LIVE_DB_KEY_DATA_TYPE_BY_DISP_ATTR[importDispLink.dispAttrName]

    # These are not defined on the tuple, they are added in DispImportController
    rawValue = importDispLink.internalRawValue
    displayValue = importDispLink.internalDisplayValue

    newLiveDbKey = ImportLiveDbItemTuple(
        dataType=dataType,
        rawValue=rawValue,
        displayValue=displayValue,
        key=importDispLink.liveDbKey,
    )

    liveDbItemsToImportByKey[importDispLink.liveDbKey] = newLiveDbKey

    return newLiveDbKey



import logging
from typing import Dict

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkUpdateHandlerABC import (
    ACIChunkUpdateHandlerABC,
)
from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_plugin_diagram._private.client.controller.BranchIndexCacheController import (
    clientBranchIndexUpdateFromServerFilt,
)
from peek_plugin_diagram._private.storage.branch.BranchIndexEncodedChunk import (
    BranchIndexEncodedChunk,
)

logger = logging.getLogger(__name__)


class BranchIndexChunkUpdateHandler(ACIChunkUpdateHandlerABC):
    _ChunkedTuple: ACIEncodedChunkTupleABC = BranchIndexEncodedChunk
    _updateFromServerFilt: Dict = clientBranchIndexUpdateFromServerFilt
    _logger: logging.Logger = logger



import logging

from sqlalchemy import Column, BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Integer, String
from sqlalchemy.orm import relationship
from sqlalchemy.sql.schema import Index
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import (
    ACIProcessorQueueTupleABC,
)
from peek_plugin_base.storage.TypeDecorators import PeekLargeBinary
from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from .DeclarativeBase import DeclarativeBase
from .Display import DispBase
from .ModelSet import ModelCoordSet

logger = logging.getLogger(__name__)


@addTupleType
class GridKeyCompilerQueue(Tuple, DeclarativeBase, ACIProcessorQueueTupleABC):
    __tablename__ = "GridKeyCompilerQueue"
    __tupleType__ = diagramTuplePrefix + __tablename__

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    gridKey = Column(String(30), primary_key=True)
    coordSetId = Column(
        Integer, ForeignKey("ModelCoordSet.id", ondelete="CASCADE"), primary_key=True
    )

    __table_args__ = (
        Index("idx_GKCompQueue_coordSetId_gridKey", coordSetId, gridKey, unique=False),
    )

    @classmethod
    def sqlCoreLoad(cls, row):
        return GridKeyCompilerQueue(
            id=row.id, coordSetId=row.coordSetId, gridKey=row.gridKey
        )

    @property
    def ckiUniqueKey(self):
        return self.gridKey


@addTupleType
class GridKeyIndex(Tuple, DeclarativeBase):
    __tablename__ = "GridKeyIndex"
    __tupleType__ = diagramTuplePrefix + __tablename__

    gridKey = Column(String(30), primary_key=True)
    dispId = Column(
        BigInteger, ForeignKey("DispBase.id", ondelete="CASCADE"), primary_key=True
    )

    disp = relationship(DispBase)

    coordSetId = Column(
        Integer, ForeignKey("ModelCoordSet.id", ondelete="CASCADE"), nullable=False
    )
    coordSet = relationship(ModelCoordSet)

    __table_args__ = (
        Index("idx_GridKeyIndex_gridKey", gridKey, unique=False),
        Index("idx_GridKeyIndex_dispId", dispId, unique=False),
        Index("idx_GridKeyIndex_coordSetId", coordSetId, unique=False),
    )


@addTupleType
class GridKeyIndexCompiled(Tuple, DeclarativeBase, ACIEncodedChunkTupleABC):
    __tablename__ = "GridKeyIndexCompiled"
    __tupleType__ = diagramTuplePrefix + __tablename__

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    gridKey = Column(String(30), nullable=False)
    encodedGridTuple = Column(PeekLargeBinary, nullable=False)
    lastUpdate = Column(String(50), nullable=False)

    coordSetId = Column(
        Integer, ForeignKey("ModelCoordSet.id", ondelete="CASCADE"), nullable=False
    )
    coordSet = relationship(ModelCoordSet)

    __table_args__ = (
        Index("idx_GKIndexUpdate_coordSetId", coordSetId, unique=False),
        Index("idx_GKIndexUpdate_gridKey", gridKey, unique=True),
    )

    @property
    def ckiChunkKey(self):
        return self.gridKey

    @property
    def ckiHasEncodedData(self) -> bool:
        return bool(self.encodedGridTuple)

    @property
    def ckiLastUpdate(self):
        return self.lastUpdate

    @classmethod
    def ckiCreateDeleteEncodedChunk(cls, chunkKey: str):
        from peek_plugin_diagram._private.tuples.grid.EncodedGridTuple import (
            EncodedGridTuple,
        )

        return EncodedGridTuple(gridKey=chunkKey)

    @classmethod
    def sqlCoreChunkKeyColumn(cls):
        return cls.__table__.c.gridKey

    @classmethod
    def sqlCoreLoad(cls, row):
        from peek_plugin_diagram._private.tuples.grid.EncodedGridTuple import (
            EncodedGridTuple,
        )

        return EncodedGridTuple(
            gridKey=row.gridKey,
            encodedGridTuple=row.encodedGridTuple,
            lastUpdate=row.lastUpdate,
        )



import logging

from peek_abstract_chunked_index.private.client.controller.ACICacheControllerABC import (
    ACICacheControllerABC,
)
from peek_plugin_diagram._private.PluginNames import diagramFilt
from peek_plugin_diagram._private.server.client_handlers.ClientLocationIndexLoaderRpc import (
    ClientLocationIndexLoaderRpc,
)
from peek_plugin_diagram._private.tuples.location_index.EncodedLocationIndexTuple import (
    EncodedLocationIndexTuple,
)

logger = logging.getLogger(__name__)

clientLocationIndexUpdateFromServerFilt = dict(
    key="clientLocationIndexUpdateFromServer"
)
clientLocationIndexUpdateFromServerFilt.update(diagramFilt)


class LocationIndexCacheController(ACICacheControllerABC):
    """Disp Key Cache Controller

    The encodedChunk cache controller stores all the locationIndexs in memory, allowing fast access from
    the mobile and desktop devices.

    """

    _ChunkedTuple = EncodedLocationIndexTuple
    _chunkLoadRpcMethod = ClientLocationIndexLoaderRpc.loadLocationIndexes
    _updateFromServerFilt = clientLocationIndexUpdateFromServerFilt
    _logger = logger



from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import (
    ACIProcessorQueueTupleABC,
)
from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from peek_plugin_diagram._private.storage.DeclarativeBase import DeclarativeBase
from sqlalchemy import Column, Index, BigInteger
from vortex.Tuple import addTupleType, Tuple


@addTupleType
class DispIndexerQueue(Tuple, DeclarativeBase, ACIProcessorQueueTupleABC):
    __tablename__ = "DispCompilerQueue"
    __tupleType__ = diagramTuplePrefix + __tablename__

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    dispId = Column(BigInteger, primary_key=True)

    __table_args__ = (Index("idx_DispCompQueue_dispId", dispId, unique=False),)

    @classmethod
    def sqlCoreLoad(cls, row):
        return DispIndexerQueue(id=row.id, dispId=row.dispId)

    @property
    def ckiUniqueKey(self):
        return self.dispId



from typing import List

from peek_plugin_diagram._private.PluginNames import diagramTuplePrefix
from vortex.Tuple import addTupleType, TupleField, Tuple


@addTupleType
class DispKeyLocationTuple(Tuple):
    __tupleType__ = diagramTuplePrefix + "DispKeyLocationTuple"

    coordSetKey: str = TupleField()

    coordSetId: int = TupleField()
    dispId: int = TupleField()

    x: float = TupleField()
    y: float = TupleField()

    def toLocationJson(self) -> str:
        return "[%s,%s,%s,%s]" % (self.coordSetId, self.dispId, self.x, self.y)

    @classmethod
    def fromLocationJson(self, items: List[int]) -> "DispKeyLocationTuple":
        assert len(items) == 4, "Invalid packed data."
        newItem = DispKeyLocationTuple()

        newItem.coordSetId = items[0]
        newItem.dispId = items[1]
        newItem.x = items[2]
        newItem.y = items[3]

        return newItem



"""Added edgeColor removed lu indexes

Peek Plugin Database Migration Script

Revision ID: da424430bf97
Revises: 473d5f6a97e8
Create Date: 2019-06-30 16:24:37.618354

"""

# revision identifiers, used by Alembic.
revision = "da424430bf97"
down_revision = "473d5f6a97e8"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_DispEllipse_fillColorId", table_name="DispEllipse", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispEllipse_lineColorId", table_name="DispEllipse", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispEllipse_lineStyleId", table_name="DispEllipse", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispPolygon_fillColorId", table_name="DispPolygon", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispPolygon_lineColorId", table_name="DispPolygon", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispPolygon_lineStyleId", table_name="DispPolygon", schema="pl_diagram"
    )
    op.add_column(
        "DispPolyline",
        sa.Column("edgeColorId", sa.Integer(), nullable=True),
        schema="pl_diagram",
    )
    op.drop_index(
        "idx_DispPolyline_lineColorId", table_name="DispPolyline", schema="pl_diagram"
    )
    op.drop_index(
        "idx_DispPolyline_lineStyleId", table_name="DispPolyline", schema="pl_diagram"
    )
    op.create_foreign_key(
        "DispGroupPointer_edgeColorId_fkey",
        "DispPolyline",
        "DispColor",
        ["edgeColorId"],
        ["id"],
        source_schema="pl_diagram",
        referent_schema="pl_diagram",
    )
    op.drop_index("idx_DispText_colorId", table_name="DispText", schema="pl_diagram")
    op.drop_index("idx_DispText_styleId", table_name="DispText", schema="pl_diagram")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "idx_DispText_styleId",
        "DispText",
        ["textStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispText_colorId",
        "DispText",
        ["colorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.drop_constraint(
        "DispGroupPointer_edgeColorId_fkey",
        "DispPolyline",
        schema="pl_diagram",
        type_="foreignkey",
    )
    op.create_index(
        "idx_DispPolyline_lineStyleId",
        "DispPolyline",
        ["lineStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolyline_lineColorId",
        "DispPolyline",
        ["lineColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.drop_column("DispPolyline", "edgeColorId", schema="pl_diagram")
    op.create_index(
        "idx_DispPolygon_lineStyleId",
        "DispPolygon",
        ["lineStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolygon_lineColorId",
        "DispPolygon",
        ["lineColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispPolygon_fillColorId",
        "DispPolygon",
        ["fillColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispEllipse_lineStyleId",
        "DispEllipse",
        ["lineStyleId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispEllipse_lineColorId",
        "DispEllipse",
        ["lineColorId"],
        unique=False,
        schema="pl_diagram",
    )
    op.create_index(
        "idx_DispEllipse_fillColorId",
        "DispEllipse",
        ["fillColorId"],
        unique=False,
        schema="pl_diagram",
    )
    # ### end Alembic commands ###



import logging
from typing import Union

from twisted.internet.defer import Deferred, inlineCallbacks

from peek_plugin_diagram._private.client.controller.GridCacheController import (
    GridCacheController,
)
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

logger = logging.getLogger(__name__)


class GridCacheIndexTupleProvider(TuplesProviderABC):
    def __init__(self, gridCacheController: GridCacheController):
        self._gridCacheController = gridCacheController

    @inlineCallbacks
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        tuples = [
            [i[0], i[1]]
            for i in self._gridCacheController.encodedChunkLastUpdateByKey().items()
        ]
        sorted(tuples, key=lambda i: i[0])

        start = tupleSelector.selector.get("start")
        count = tupleSelector.selector.get("count")

        if start is not None and count:
            tuples = tuples[start:count]

        payloadEnvelope = yield Payload(filt, tuples=tuples).makePayloadEnvelopeDefer()
        vortexMsg = yield payloadEnvelope.toVortexMsgDefer()
        return vortexMsg
